from msegat.handlers.handler import Handler
from msegat.helpers.api_helper import APIHelper
from msegat.responses.gw import send_sms as response
from msegat.requests.gw import send_sms as requests


class SendSMSHandler(Handler):
    """
    This API is used to send a sms to one or more recipient
    """

    path = '/gw/sendsms.php'
    response_model = response.SendSMSResponse

    def __call__(self, request: requests.SendSMSRequest):
        """
        In Python, classes, methods, and instances are callable because calling a class returns a new instance.
        Instances are callable if their class includes __call__() method.
        """
        parameters = request.to_dictionary()
        request = self.http_client.post(self.path, parameters=parameters)
        context = self.execute_request(request)
        return APIHelper.json_deserialize(context.response.raw_body, self.response_model.form_dictionary)


send_sms_handler = SendSMSHandler()



from msegat.configuration import Configuration


class SendSMSRequest(object):
    """
    A class to represent an API request for sending an SMS to one or more recipients.
    """
    USER_NAME = 'userName'
    NUMBERS = 'numbers'
    USER_SENDER = 'userSender'
    API_Key = 'apiKey'
    MSG = 'msg'

    def __init__(self, numbers: str, msg: str):
        """
        Initialize a new instance of SendSMSRequest.

        :param numbers: A comma-separated string of phone numbers.
        :param msg: The message content to be sent.
        """
        self.user_name = Configuration.user_name
        self.user_sender = Configuration.user_sender
        self.api_key = Configuration.api_key
        self.numbers = numbers
        self.msg = msg

    def to_dictionary(self):
        """
        Convert the instance attributes into a dictionary.

        :return: A dictionary representation of the SendSMSRequest instance.
        """
        return {
            self.USER_NAME: self.user_name,
            self.NUMBERS: self.numbers,
            self.USER_SENDER: self.user_sender,
            self.API_Key: self.api_key,
            self.MSG: self.msg
        }

    @classmethod
    def from_dictionary(cls, dictionary: dict):
        """
        Create an instance of SendSMSRequest from a dictionary.

        :param dictionary: A dictionary with keys 'numbers' and 'msg'.
        :return: An instance of SendSMSRequest or None if the dictionary is invalid.
        """
        if dictionary is None:
            return None

        # Extract values from the dictionary
        numbers = dictionary.get(cls.NUMBERS)
        msg = dictionary.get(cls.MSG)

        # Return a new SendSMSRequest instance
        return cls(numbers=numbers, msg=msg)



class HttpResponse(object):
    """
    Information about an HTTP Response including its status code, returned
    headers, and raw body
    """

    def __init__(self, status_code, headers, raw_body):
        """
        When you create a new object of a class, Python automatically calls the __init__() method to
        initialize the object’s attributes.
        """
        self.status_code = status_code
        self.headers = headers
        self.raw_body = raw_body



from msegat.http.http_method_enum import HttpMethodEnum
from msegat.http.http_request import HttpRequest


class HttpClient(object):
    """
    An interface for the methods that an HTTP Client must implement
    This class should not be instantiated but should be used as a base class
    for HTTP Client classes.
    """

    def execute(self, request):
        """
        Execute a given HttpRequest to get a string response back
        """
        raise NotImplementedError("Please Implement this method")

    def convert_response(self, response, binary):
        """
        Converts the Response object of the HttpClient into an
        HttpResponse object.
        """
        raise NotImplementedError("Please Implement this method")

    def get(self, query_url, headers=None, query_parameters=None):
        """
        Create a simple GET HttpRequest object for the given parameters
        """

        return HttpRequest(HttpMethodEnum.GET, query_url, headers, query_parameters, None, None)

    def head(self, query_url, headers=None, query_parameters=None):
        """
        Create a simple HEAD HttpRequest object for the given parameters
        """
        return HttpRequest(HttpMethodEnum.HEAD, query_url, headers, query_parameters, None, None)

    def post(self, query_url, headers=None, query_parameters=None, parameters=None, files=None):
        """
        Create a simple POST HttpRequest object for the given parameters
        """
        return HttpRequest(HttpMethodEnum.POST, query_url, headers, query_parameters, parameters, files)

    def put(self, query_url, headers=None, query_parameters=None, parameters=None, files=None):
        """
        Create a simple PUT HttpRequest object for the given parameters
        """
        return HttpRequest(HttpMethodEnum.PUT, query_url, headers, query_parameters, parameters, files)

    def patch(self, query_url, headers=None, query_parameters=None, parameters=None, files=None):
        """
        Create a simple PATCH HttpRequest object for the given parameters
        """
        return HttpRequest(HttpMethodEnum.PATCH, query_url, headers, query_parameters, parameters, files)

    def delete(self, query_url, headers=None, query_parameters=None, parameters=None, files=None):
        """
        Create a simple DELETE HttpRequest object for the given parameters
        """
        return HttpRequest(HttpMethodEnum.DELETE, query_url, headers, query_parameters, parameters, files)



from msegat.configuration import Configuration
from msegat.requests.gw.send_sms import SendSMSRequest
from msegat.handlers.msgs.send_sms import send_sms_handler


class MsegatClient(object):
    """
    A client class for interacting with the Msegat SMS service API.
    """
    config = Configuration

    def __init__(self, user_sender, user_name, api_key):
        """
        Initializes the MsegatClient with the required credentials.

        :param user_sender: The sender's identifier (e.g., phone number or username).
        :param user_name: The username for authentication.
        :param api_key: The API token for authenticating requests.
        """
        Configuration.user_sender = user_sender
        Configuration.user_name = user_name
        Configuration.api_key = api_key

    def send_message(self, numbers, msg):
        """
        Sends an SMS message to one or more recipients via the Msegat API.

        :param numbers: A comma-separated string of recipient phone numbers.
                        Can send to one or multiple numbers (up to 500 per request).
        :param msg: The content of the SMS message. The system automatically handles encoding for different languages.

        :return: An HTTP response. If successful, returns status 200 with the result in the response body as JSON.
                 On failure, returns an HTTP error response with a JSON body containing an error code and description.
        """
        return send_sms_handler(request=SendSMSRequest(numbers, msg))




import re
import sys
import datetime
import calendar
import email.utils as eut
from time import mktime

import jsonpickle
import dateutil.parser
from urllib.parse import quote


class APIHelper(object):
    """
    A Helper Class for various functions associated with API Calls.
    This class contains static methods for operations that need to be
    performed during API requests. All the methods inside this class are
    static methods, there is no need to ever initialise an instance of this
    class.

    """

    @staticmethod
    def merge_dicts(dict1, dict2):
        """
        Merges two dictionaries into one as a shallow copy.
        """
        dict1 = dict1 if dict1 else {}
        dict2 = dict2 if dict2 else {}
        temp = dict1.copy()
        temp.update(dict2)
        return temp

    @staticmethod
    def json_serialize(obj):
        """
        JSON Serialization of a given object.
        """
        if obj is None:
            return None

        if isinstance(obj, list):
            value = list()
            for item in obj:
                if hasattr(item, "_names"):
                    value.append(APIHelper.to_dictionary(item))
                else:
                    value.append(item)
            obj = value
        else:
            if hasattr(obj, "_names"):
                obj = APIHelper.to_dictionary(obj)

        return jsonpickle.encode(obj, False)

    @staticmethod
    def json_deserialize(json, unboxing_function=None):
        """
        JSON Deserialization of a given string.
        """
        if json is None:
            return None

        try:
            decoded = jsonpickle.decode(json)
        except (Exception,):
            return json

        if unboxing_function is None:
            return decoded
        else:
            return unboxing_function(decoded)

    @staticmethod
    def serialize_array(key, array, formatting="indexed"):
        """
        Converts an array parameter to a list of key value tuples.
        """
        tuples = []

        if sys.version_info[0] < 3:
            serializable_types = (str, int, int, float, bool, datetime.date, APIHelper.CustomDate)
        else:
            serializable_types = (str, int, float, bool, datetime.date, APIHelper.CustomDate)

        if isinstance(array[0], serializable_types):
            if formatting == "unindexed":
                tuples += [("{0}[]".format(key), element) for element in array]
            elif formatting == "indexed":
                tuples += [("{0}[{1}]".format(key, index), element) for index, element in enumerate(array)]
            elif formatting == "plain":
                tuples += [(key, element) for element in array]
            else:
                raise ValueError("Invalid format provided.")
        else:
            tuples += [("{0}[{1}]".format(key, index), element) for index, element in enumerate(array)]

        return tuples

    @staticmethod
    def append_url_with_template_parameters(url, parameters, encode=True):
        """
        Replaces template parameters in the given url.
        """

        if url is None:
            raise ValueError("URL is None.")
        if parameters is None:
            return url

        for key in parameters:
            element = parameters[key]
            replace_value = ''
            if element is None:
                replace_value = ''
            elif isinstance(element, list):
                replace_value = "/".join((quote(str(x), safe='') if encode else str(x)) for x in element)
            else:
                replace_value = quote(str(element), safe='') if encode else str(element)

            url = url.replace('{{{0}}}'.format(key), str(replace_value))

        return url

    @staticmethod
    def append_url_with_query_parameters(url, parameters=None, array_serialization="indexed"):
        """
        Adds query parameters to a URL.
        """

        if url is None:
            raise ValueError("URL is None.")
        if parameters is None:
            return url

        for key, value in parameters.items():
            seperator = '&' if '?' in url else '?'
            if value is not None:
                if isinstance(value, list):
                    value = [element for element in value if element]
                    if array_serialization == "csv":
                        url += "{0}{1}={2}".format(seperator, key,
                                                   ",".join(quote(str(x), safe='') for x in value))
                    elif array_serialization == "psv":
                        url += "{0}{1}={2}".format(seperator, key,
                                                   "|".join(quote(str(x), safe='') for x in value))
                    elif array_serialization == "tsv":
                        url += "{0}{1}={2}".format(seperator, key,
                                                   "\t".join(quote(str(x), safe='') for x in value))
                    else:
                        url += "{0}{1}".format(seperator,
                                               "&".join(("{0}={1}".format(k, quote(str(v), safe='')))
                                                        for k, v in
                                                        APIHelper.serialize_array(key, value, array_serialization)))
                else:
                    url += "{0}{1}={2}".format(seperator, key, quote(str(value), safe=''))

        return url

    @staticmethod
    def clean_url(url):
        """
        Validates and processes the given query Url to clean empty slashes.
        """

        regex = "^https?://[^/]+"
        match = re.match(regex, url)
        if match is None:
            raise ValueError('Invalid Url format.')

        protocol = match.group(0)
        index = url.find('?')
        query_url = url[len(protocol): index if index != -1 else None]
        query_url = re.sub("//+", "/", query_url)
        parameters = url[index:] if index != -1 else ""

        return protocol + query_url + parameters

    @staticmethod
    def form_encode_parameters(form_parameters, array_serialization="indexed"):
        """
        Form encodes a dictionary of form parameters
        """
        encoded = []

        for key, value in form_parameters.items():
            encoded += APIHelper.form_encode(value, key, array_serialization)

        return encoded

    @staticmethod
    def form_encode(obj, instance_name, array_serialization="indexed"):
        """
        Encodes a model in a form-encoded manner such as person[Name]
        """
        retval = []

        if hasattr(obj, "_names"):
            obj = APIHelper.to_dictionary(obj)

        if obj is None:
            return []
        elif isinstance(obj, list):
            for element in APIHelper.serialize_array(instance_name, obj, array_serialization):
                retval += APIHelper.form_encode(element[1], element[0], array_serialization)
        elif isinstance(obj, dict):
            for item in obj:
                retval += APIHelper.form_encode(obj[item], instance_name + "[" + item + "]", array_serialization)
        else:
            retval.append((instance_name, obj))

        return retval

    @staticmethod
    def to_dictionary(obj):
        """
        Creates a dictionary representation of a class instance. The
        keys are taken from the API description and may differ from language
        specific variable names of properties.
        """
        dictionary = dict()

        for name in obj._names:
            value = getattr(obj, name)
            if isinstance(value, list):
                # Loop through each item
                dictionary[obj._names[name]] = list()
                for item in value:
                    dictionary[obj._names[name]].append(
                        APIHelper.to_dictionary(item) if hasattr(item, "_names") else item)
            elif isinstance(value, dict):
                dictionary[obj._names[name]] = dict()
                for key in value:
                    dictionary[obj._names[name]][key] = APIHelper.to_dictionary(value[key]) if hasattr(value[key],
                                                                                                       "_names") else \
                    value[key]
            else:
                dictionary[obj._names[name]] = APIHelper.to_dictionary(value) if hasattr(value, "_names") else value

        # Return the result
        return dictionary

    @staticmethod
    def when_defined(func, value):
        return func(value) if value else None

    class CustomDate(object):
        """
        A base class for wrapper classes of datetime.
        This class contains methods which help in
        appropriate serialization of datetime objects.
        """

        def __init__(self, dtime, value=None):
            self.datetime = dtime
            if not value:
                self.value = self.from_datetime(dtime)
            else:
                self.value = value

        def __str__(self):
            return self.value

        def __getstate__(self):
            return self.value

        def __setstate__(self, state):
            pass

    class HttpDateTime(CustomDate):
        """
        A wrapper class for datetime to support HTTP date format.
        """

        @classmethod
        def from_datetime(cls, date_time):
            return eut.formatdate(timeval=mktime(date_time.timetuple()),
                                  localtime=False, usegmt=True)

        @classmethod
        def from_value(cls, value):
            dtime = datetime.datetime.fromtimestamp(eut.mktime_tz(eut.parsedate_tz(value)))
            return cls(dtime, value)

    class UnixDateTime(CustomDate):
        """
        A wrapper class for datetime to support Unix date format.
        """

        def __str__(self):
            return str(self.value)

        @classmethod
        def from_datetime(cls, date_time):
            return calendar.timegm(date_time.utctimetuple())

        @classmethod
        def from_value(cls, value):
            dtime = datetime.datetime.utcfromtimestamp(float(value))
            return cls(dtime, float(value))

    class RFC3339DateTime(CustomDate):
        """
        A wrapper class for datetime to support Rfc 3339 format.
        """

        @classmethod
        def from_datetime(cls, date_time):
            return date_time.isoformat()

        @classmethod
        def from_value(cls, value):
            dtime = dateutil.parser.parse(value)
            return cls(dtime, value)



class Base(object):
    """
    This class should not be instantiated but rather serves as a fundamental base class for
    deriving specialized implementations.
    """
    properties = {}

    @classmethod
    def form_dict(cls, data: dict):
        """
        Create and return a new object.
        """
        raise NotImplementedError("Please Implement this method")

    @classmethod
    def getmembers(cls):
        """
        This module provides a method called getmemebers()
        that returns a list of class attributes and methods.
        """
        raise NotImplementedError("Please Implement this method")

    def to_dict(self):
        """
        Return the property of the object as a dictionary
        """
        raise NotImplementedError("Please Implement this method")



class APITokenAuthentication(object):

    @staticmethod
    def apply(http_request):
        """
        Add token authentication to the header request.
        Args:
            http_request (HttpRequest): The HttpRequest object to which authentication will be added.
        """
        pass



from msegat.client import MsegatClient


def send_sms():
    """
    This API is used to send a sms to one or more recipient
    """
    user_sender = input("Please Enter the user sender:")
    user_name = input("Please Enter the user name:")
    api_key = input("Please Enter the apiKey:")
    numbers = input("Please Enter the numbers:")
    msg = input("Please Enter the msg:")
    return MsegatClient(user_sender, user_name, api_key).send_message(numbers, msg)


if __name__ == '__main__':
    send_sms()



import requests

from requests.adapters import HTTPAdapter
from requests.packages.urllib3.util.retry import Retry

from msegat.http.http_client import HttpClient
from msegat.http.http_method_enum import HttpMethodEnum
from msegat.http.http_response import HttpResponse


class RequestsClient(HttpClient):
    """
    An implementation of HttpClient that uses Requests as its HTTP Client
    """

    def __init__(self, timeout=60, cache=False, max_retries=None, retry_interval=None):
        """
        When you create a new object of a class, Python automatically calls the __init__() method to
        initialize the object’s attributes.
        """
        self.timeout = timeout
        self.session = requests.session()

        if max_retries and retry_interval:
            retries = Retry(total=max_retries, backoff_factor=retry_interval)
            self.session.mount('http://', HTTPAdapter(max_retries=retries))
            self.session.mount('https://', HTTPAdapter(max_retries=retries))

    def execute(self, request):
        """
        Execute a given HttpRequest to get a string response back
        """
        response = self.session.request(
            HttpMethodEnum.to_string(request.http_method), request.query_url,
            headers=request.headers, params=request.query_parameters,
            data=request.parameters, files=request.files, timeout=self.timeout
        )
        return self.convert_response(response)

    def convert_response(self, response):
        """
        Converts the Response object of the HttpClient into an
        HttpResponse object.
        """
        return HttpResponse(response.status_code, response.headers, response.text)



from msegat.helpers.api_helper import APIHelper


class HttpRequest(object):
    """
    Information about an HTTP Request including its method, headers,
    parameters, URL, and Basic Auth details
    """

    def __init__(self, http_method, query_url, headers=None, query_parameters=None, parameters=None, files=None):
        """
        Constructor for the HttpRequest class
        """
        self.http_method = http_method
        self.query_url = query_url
        self.headers = headers
        self.query_parameters = query_parameters
        self.parameters = parameters
        self.files = files

    def add_header(self, name, value):
        """
        Add a header to the HttpRequest.
        """
        self.headers[name] = value

    def add_parameter(self, name, value):
        """
        Add a parameter to the HttpRequest.
        """
        self.parameters[name] = value

    def add_query_parameter(self, name, value):
        """
        Add a query parameter to the HttpRequest.
        """
        self.query_url = APIHelper.append_url_with_query_parameters(self.query_url,
                                                                    {name: value})
        self.query_url = APIHelper.clean_url(self.query_url)



class APIException(Exception):
    """
    Class that handles HTTP Exceptions when fetching API Endpoints.
    """

    def __init__(self, message, context):
        """
        When you create a new object of a class, Python automatically calls the __init__() method to
        initialize the object’s attributes.
        """
        super(APIException, self).__init__(message)
        self.context = context
        self.response_code = context.response.status_code



class Configuration(object):
    """
    This class is designed for configuring the SDK by the user. It does not require instantiation,
    and all properties and methods can be accessed without creating an instance.
    """
    # An enum for API servers
    class Server(object):
        BASE_URL = 'base_url'

    # An enum for SDK environments
    class Environment(object):
        """
        Describe the setting where software and other products are actually put
        into operation for their intended uses by end users
        """
        PRODUCTION = 'production'

    # All the environments the SDK can run in
    environments = {
        Environment.PRODUCTION: {
            Server.BASE_URL: 'https://www.msegat.com',
        },
    }

    # The environment in which the SDK is running
    environment = Environment.PRODUCTION

    # The API token to use with basic authentication
    api_key = None

    # The sender's user information (e.g., email or username)
    user_sender = None

    # The username for authentication or reference
    user_name = None

    @classmethod
    def get_base_uri(cls, server=Server.BASE_URL):
        """
        This function generates the appropriate base URI for the given environment and server.
        It plays a crucial role in determining the foundation for all API requests and interactions.
        By dynamically creating the base URI, it ensures that the SDK operates seamlessly in various environments,
        adapting to the specific server requirements while maintaining a consistent and reliable connection.
        """
        return cls.environments[cls.environment][server]



class HttpContext(object):
    """
    An HTTP Context that contains both the original HttpRequest
    object that initiated the call and the HttpResponse object that
    is the result of the call.
    """

    def __init__(self, request, response):
        """
        Constructor for the HttpContext class
        """
        self.request = request
        self.response = response



import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

classifiers = [
    "Programming Language :: Python :: 3",
    "License :: OSI Approved :: MIT License",
    "Operating System :: OS Independent",
]

requires = [
    "requests>=2.31.0",
    "python-dateutil>=2.8.2",
    "jsonpickle>=3.0.2",
    "urllib3>=1.26",
    "six>=1.16.0",
]

setuptools.setup(
    name="msegat",
    version="1.0.0",
    description="Msegat API client",
    author="Abdullah Alaidrous",
    author_email="abd.alaidrous@gmail.com",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://gitlab.com/abdroos/msegat",
    package_dir={"msegat": "msegat"},
    packages=setuptools.find_packages(),
    python_requires=">=3.6",
    install_requires=requires,
    classifiers=classifiers,
)



import json
from msegat.configuration import Configuration
from msegat.exceptions.api_exception import APIException
from msegat.helpers.api_helper import APIHelper
from msegat.http.auth.api_token import APITokenAuthentication
from msegat.http.http_context import HttpContext
from msegat.http.requests_client import RequestsClient


class Handler(object):
    """
    All handlers inherit from this base class.
    """

    path = ''
    authentication_model = APITokenAuthentication

    def __init__(self, client=None):
        """
        When you create a new object of a class, Python automatically calls the __init__() method to
        initialize the object’s attributes.
        """
        self.global_headers = {'Content-Type': 'application/json'}
        self.base_url = Configuration.get_base_uri(Configuration.Server.BASE_URL)
        self.http_client = client if client else RequestsClient()

    def validate_response(self, context):
        """
        Validates an HTTP response by checking for global errors.
        """

        print(context.response.raw_body)

        if context.response.status_code == 404:
            raise APIException('Page not Found', context)
        if context.response.status_code == 401:
            raise APIException('Authentication failed', context)
        elif context.response.status_code == 406:
            raise APIException('Wrong parameter format', context)
        elif context.response.status_code == 449:
            raise APIException('Message body is empty', context)
        elif context.response.status_code == 451:
            raise APIException('TimeScheduled parameter must indicate time in the future', context)
        elif context.response.status_code == 480:
            raise APIException('This user cannot use specified SenderID', context)
        elif context.response.status_code == 482:
            raise APIException('Invalid dest num', context)
        elif context.response.status_code < 200 or context.response.status_code > 208:
            raise APIException('HTTP response not OK.', context)

        return context

    def execute_request(self, request):
        """
        Executes an HttpRequest.
        """
        request.query_url = self.base_url + request.query_url
        request.headers = APIHelper.merge_dicts(self.global_headers, request.headers)
        request.parameters = json.dumps(request.parameters)
        self.authentication_model.apply(request)
        response = self.http_client.execute(request)
        context = HttpContext(request, response)
        self.validate_response(context)
        return context



class SendSMSResponse(object):
    """
    This API is used to send a message to one or more recipients
    """

    def to_dictionary(self):
        """
        Return the property of the object as a dictionary
        """
        return {}

    @classmethod
    def form_dictionary(cls, dictionary: dict):
        """
        Creates an instance of this model from a dictionary
        """
        if dictionary is None:
            return None

        # Return an object of this model
        return cls()
